
using System;
using System.Collections.Generic;
using System.Text;
using Esri.ArcGISRuntime.Geometry;
using Esri.ArcGISRuntime.Mapping;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;
using Esri.ArcGISRuntime.Portal;

using Esri.ArcGISRuntime.Symbology;
using Esri.ArcGISRuntime.Tasks.Offline;
using Esri.ArcGISRuntime.UI;
using System.Windows;
using System.Diagnostics;
using System.Drawing;

namespace DisplayAnOfflineMapOnDemand
{
    class MapViewModel : INotifyPropertyChanged
    {
        public MapViewModel()
        {
            SetupMap();
        }
     
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private Map _map;
        public Map Map
        {
            get { return _map; }
            set
            {
                _map = value;
                OnPropertyChanged();
            }
        }

        private GraphicsOverlayCollection _graphicsOverlays;
        public GraphicsOverlayCollection GraphicsOverlays
        {
            get { return _graphicsOverlays; }
            set
            {
                _graphicsOverlays = value;
                OnPropertyChanged();
            }
        }

        private async Task SetupMap()
        {
            // Create a portal pointing to ArcGIS Online.
            ArcGISPortal portal = await ArcGISPortal.CreateAsync();
            
            // Create a portal item for a specific web map id.

            string webMapId = "acc027394bc84c2fb04d1ed317aac674";

            PortalItem mapItem = await PortalItem.CreateAsync(portal, webMapId);

            // Create the map from the item.
            Map map = new Map(mapItem);

            // Set the view model "Map" property.
            this.Map = map;

            // Define area of interest (envelope) to take offline.
            EnvelopeBuilder envelopeBldr = new EnvelopeBuilder(SpatialReferences.Wgs84)
            {
                XMin = -88.1526,
                XMax = -88.1490,
                YMin = 41.7694,
                YMax = 41.7714
            };

            Envelope offlineArea = envelopeBldr.ToGeometry();

            // Create a graphic to display the area to take offline.
            SimpleLineSymbol lineSymbol = new SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.Red, 2);
            SimpleFillSymbol fillSymbol = new SimpleFillSymbol(SimpleFillSymbolStyle.Solid, Color.Transparent, lineSymbol);
            Graphic offlineAreaGraphic = new Graphic(offlineArea, fillSymbol);

            // Create a graphics overlay and add the graphic.
            GraphicsOverlay areaOverlay = new GraphicsOverlay();
            areaOverlay.Graphics.Add(offlineAreaGraphic);
            
            // Add the overlay to a new graphics overlay collection.
            GraphicsOverlayCollection overlays = new GraphicsOverlayCollection
            {
                areaOverlay
            };

            // Set the view model's "GraphicsOverlays" property (will be consumed by the map view).
            this.GraphicsOverlays = overlays;

            // Create an offline map task using the current map.
            OfflineMapTask offlineMapTask = await OfflineMapTask.CreateAsync(map);

            // Create a default set of parameters for generating the offline map from the area of interest.
            GenerateOfflineMapParameters parameters = await offlineMapTask.CreateDefaultGenerateOfflineMapParametersAsync(offlineArea);
            parameters.UpdateMode = GenerateOfflineMapUpdateMode.NoUpdates;

            // Build a folder path named with today's date/time in the "My Documents" folder.
            string documentsFolder = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            string downloadDirectory = System.IO.Path.Combine(documentsFolder, "OfflineMap_" + DateTime.Now.ToFileTime().ToString());

            GenerateOfflineMapJob generateJob = offlineMapTask.GenerateOfflineMap(parameters, downloadDirectory);

            generateJob.ProgressChanged += GenerateJob_ProgressChanged;

            generateJob.Start();

        }

        private async void GenerateJob_ProgressChanged(object sender, EventArgs e)
        {

            try
            {
                GenerateOfflineMapJob generateJob = (GenerateOfflineMapJob)sender;

                // If the job succeeds, show the offline map in the map view.
                if (generateJob.Status == Esri.ArcGISRuntime.Tasks.JobStatus.Succeeded)
                {
                    var result = await generateJob.GetResultAsync();
                    this.Map = result.OfflineMap;
                    Debug.WriteLine("Generate offline map: Complete");
                }
                // If the job fails, notify the user.
                else if (generateJob.Status == Esri.ArcGISRuntime.Tasks.JobStatus.Failed)
                {
                    MessageBox.Show($"Unable to generate a map for that area: {generateJob.Error.Message}");
                }
                else
                {
                    int percentComplete = generateJob.Progress;
                    Debug.WriteLine($"Generate offline map: {percentComplete}%");
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error generating offline map: {ex.Message}");
            }

        }

    }
}

